#include <SoftwareSerial.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define bridge_port PORTA // H-bridge port
#define bridge_direction DDRA // H-bridge direction
#define A2 (1 << PA0) // H-bridge output pins
#define A1 (1 << PA1) // "
#define B2 (1 << PA3) // "
#define B1 (1 << PA4) // "
#define on_delay() _delay_us(25) // PWM on time
#define off_delay() _delay_us(5) // PWM off time
#define PWM_count 100 // number of PWM cycles
#define step_count 100 // number of steps

static uint8_t count;

#define RX_PIN  10
#define TX_PIN  9
SoftwareSerial serial(RX_PIN, TX_PIN);

bool stopIt;
static uint8_t turns;

//
// A+ B+ PWM pulse
//
void pulse_ApBp() {
  clear(bridge_port, A2);
  clear(bridge_port, B2);
  set(bridge_port, A1);
  set(bridge_port, B1);
   for (count = 0; count < PWM_count; ++count) {
      set(bridge_port, A1);
      set(bridge_port, B1);
      on_delay();
      clear(bridge_port, A1);
      clear(bridge_port, B1);
      off_delay();
      }
   }
   
//
// A+ B- PWM pulse
//
void pulse_ApBm() {
  clear(bridge_port, A2);
  clear(bridge_port, B1);
  set(bridge_port, A1);
  set(bridge_port, B2);
   for (count = 0; count < PWM_count; ++count) {
      set(bridge_port, A1);
      set(bridge_port, B2);
      on_delay();
      clear(bridge_port, A1);
      clear(bridge_port, B2);
      off_delay();
      }
   }
   
//
// A- B+ PWM pulse
//
void pulse_AmBp() {
  clear(bridge_port, A1);
  clear(bridge_port, B2);
  set(bridge_port, A2);
  set(bridge_port, B1);
   for (count = 0; count < PWM_count; ++count) {
      set(bridge_port, A2);
      set(bridge_port, B1);
      on_delay();
      clear(bridge_port, A2);
      clear(bridge_port, B1);
      off_delay();
      }
   }
   
//
// A- B- PWM pulse
//
void pulse_AmBm() {
  clear(bridge_port, A1);
  clear(bridge_port, B1);
  set(bridge_port, A2);
  set(bridge_port, B2);
   for (count = 0; count < PWM_count; ++count) {
      set(bridge_port, A2);
      set(bridge_port, B2);
      on_delay();
      clear(bridge_port, A2);
      clear(bridge_port, B2);
      off_delay();
      }
   }
   
//
// clockwise step
//
void step_cw() {
   pulse_ApBp();
   pulse_AmBp();
   pulse_AmBm();
   pulse_ApBm();
   }

//
// counter-clockwise step
//
void step_ccw() {
   pulse_ApBm();
   pulse_AmBm();
   pulse_AmBp();
   pulse_ApBp();
}


void switchMode(int cmd) {
  //serial.println(cmd);
  switch(cmd){
    case 0: turnOff();
            //serial.println("off");
            break;
    case 1: goForward();
            break;
    case 2: goBackward();
            break;           
    default:
            return;
  }
}

void readSerial(){
  int value =0, input = 0;  

  if(serial.available() == 0)
  return;
  
  switch (serial.available()) {
        /* Serial.available() is the number of bytes waiting.  Convert from 
         * ASCII val to an int.  Intentional switch-case fall through below. 
        */ 
        case 3:
          value = serial.read();
          value -= 48;
          value *= 10;
        case 2:
          value += serial.read();
          value -= 48;
          value *= 10;
        case 1:
          input = 0;
          input = serial.read();
          //delay(10);       
          value += input;
          while (serial.available()){serial.read();}
          break;
        default:        
          // If >3 chars, just clear out the incoming buffer. 
          //while (serial.available())           
          return;      
      }
      //serial.print(input);
  //while (serial.available()) 
  if(input>0)     
  processSerialInput(input);
}

void processSerialInput(int value){  
  int mode;
  switch (value) {
    case 'a':
      mode = 0;
      break;
    case 'b':
      mode = 1;     
      break;
    case 'c':
      mode = 2;   
      break;
    case 'd':
      mode = 3;     
      break;
    case 'e':
      mode = 4;    
      break;
    case 'f':
      mode = 5;   
      break;
    case 'g':
      mode = 6;    
      break;
    case 'h':
      mode = 7;    
      break;
    case 'i':
      mode = 8;    
      break;
    case 'j':
      mode = 9;    
      break;    
    default:  
      mode = 0;
      break;  
  }
  switchMode(mode); 
}

void turnOff(){
  stopIt = true;
}

void goForward(){
  stopIt = false;
  static uint8_t i,j;
  for (i = 0; i < step_count; ++i) {
    //for (j = 0; j < i; ++j)
    
    if(stopIt) return;
    step_cw();    
    readSerial();   
  }
  serial.println(turns++);  
}

void goBackward(){  
  stopIt = false;
  static uint8_t i,j;
  for (i = 0; i < step_count; ++i) {
    //for (j = 0; j < i; ++j)
    if(stopIt) return;
    step_ccw();
    readSerial(); 
  }
  serial.println(turns--);
}

void setup() {
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   
   //
   // initialize bridge pins
   //
   clear(bridge_port, A1);
   output(bridge_direction, A1);
   clear(bridge_port, A2);
   output(bridge_direction, A2);
   clear(bridge_port, B1);
   output(bridge_direction, B1);
   clear(bridge_port, B2);
   output(bridge_direction, B2);

   serial.begin(9600);
   serial.println("Ready");
   goForward();
}

void loop() {
  /*static uint8_t i,j;

  for (i = 0; i < step_count; ++i) {
     for (j = 0; j < i; ++j)
        step_cw();
     for (j = 0; j < i; ++j)
        step_ccw();
  }
  serial.println("-");
  for (i = step_count; i > 0; --i) {
     for (j = 0; j < i; ++j)
        step_cw();
     for (j = 0; j < i; ++j)
        step_ccw();
  }*/
  readSerial();
  //delay(500);
}

